# $Id: lbsearch.tcl 98 2007-03-02 18:00:42Z kostix $
# Support for searching in listbox widgets for Tkabber.

namespace eval search {
	namespace eval listbox {}
}

proc search::listbox::new {lbox inst args} {
	set token state_$lbox@$inst
	variable $token
	upvar 0 $token state

	set state(-hilitecmd) [namespace current]::hilite

	array set state $args

	set state(self) $lbox

	set token
}

proc search::listbox::delete {token} {
	variable $token
	upvar 0 $token state

	array unset state
}

proc search::listbox::do_search {token what dir} {
	variable $token
	upvar 0 $token state

	set w $state(self)

	set selection_first 0
	set selection_last [$w index end]
	for {set i 0} {$i < [$w index end]} {incr i} {
	    if {[$w selection includes $i]} {
		if {$selection_first == 0} {
		    set selection_first $i
		}
		set selection_last $i
	    }
	}

	if {[string equal $dir down]} {
		set step   1
		set start1 [incr selection_last]
		set end1   [$w index end]
		set cond1  {$i <= $end1}
		set start2 0
		set end2   $selection_last
		set cond2  {$i < $end2}
	} else {
		set step   -1
		set start1 [incr selection_first -1]
		set end1   0
		set cond1  {$i >= $end1}
		set start2 [$w index end]
		set end2   $selection_first
		set cond2  {$i > $end2}
	}

	set found 0

	for {set i $start1} $cond1 {incr i $step} {
	    if {[[namespace parent]::match $what [$w get $i]]} {
		set found 1
		break
	    }
	}
	if {!$found} {
	    for {set i $start2} $cond2 {incr i $step} {
		if {[[namespace parent]::match $what [$w get $i]]} {
		    set found 1
		    break
		}
	    }
	}

	if {$found} {
		eval $state(-hilitecmd) $token $i
	}

	return $found
}

proc search::listbox::hilite {token index} {
	variable $token
	upvar 0 $token state

	set w $state(self)

	$w selection clear 0 end
	$w selection set $index
	$w see $index
}

